package api

import "github.com/gin-gonic/gin"

type CustomerApi interface {
	CustomerRegister(c *gin.Context)    //客户注册
	CustomerURegister(c *gin.Context)    //客户注册，仅仅依赖“用户名”，只判断用户名是否存在，不做任何其他的判断
	CustomerLogin(c *gin.Context)       //客户登录
	CustomerEdit(c *gin.Context)        //客户资料编辑
	CustomerLogout(c *gin.Context)      //客户登出
	CustomerSubInfo(c *gin.Context)     //客户订阅信息
	CustomerSubscribe(c *gin.Context)   //客户发起订阅
	CustomerUnsubscribe(c *gin.Context) //客户取消订阅
	CustomerList(c *gin.Context)        //客户列表(需权限检查)
}
